#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.


from .can_post_story import CanPostStory
from .delete_stories import DeleteStories
from .edit_story import EditStory
from .forward_story import ForwardStory
from .get_chat_active_stories import GetChatActiveStories
from .get_chat_archived_stories import GetChatArchivedStories
from .get_stories import GetStories
from .hide_my_story_view import HideMyStoryView
from .post_story import PostStory
from .toggle_story_is_posted_to_chat_page import ToggleStoryIsPostedToChatPage

class Stories(
    CanPostStory,
    DeleteStories,
    EditStory,
    ForwardStory,
    GetChatActiveStories,
    GetChatArchivedStories,
    GetStories,
    HideMyStoryView,
    PostStory,
    ToggleStoryIsPostedToChatPage,
):
    pass
